
##########################################
### SENSITIVITY STUDY WITH UNCERTAINTY ###
### IN SENSITIVITY PARAMETERS          ###
### SELECTION BIAS EXAMPLE             ###
##########################################

# Load libraries and functions ----
library(rstan)
library(msm)
source("DGM.R")
source("simulator_functions.R")

# Parameters for sensitivity study -----
agepred <- 50 # age to predict to, in units of half-years
ntestdata <- 200 # sample size

# priors 
bmu0 <- -4
bsig0 <- 2

lmumean0 <- .9
lmusig0 <- .2


# simulated data - correct voltage distribution -----
simdat <- gen_simdat(agepred)
dist_true <- simdat$volt

# generate surveillance data with selection bias ----
testdata <- gen_testdata()

# fit model without addressing selection bias ----
stan_dat <- list(y=testdata$volt,
                 age=testdata$age,
                 n=nrow(testdata))
fit0 <- stan(file = 'Linear_Model.stan', data = stan_dat,
            iter = 2000, thin =5, chains = 2)
samps0 <- extract(fit0)

# predict voltage distribution without addressing selection bias ----
testdata2 <- testdata
testdata2$age <- agepred
dist0 <- t(sapply(1:length(samps0$beta0), function(jj)
  sort(rnorm(nrow(testdata2), samps0$beta0[jj] +samps0$beta1[jj]*testdata2$age, samps0$sigma[jj]))))
dist0_summary <- make_distqs(dist0)


#sensitivity study model addressing selection bias-----
stan_dat <- list(y=testdata$volt,
                     age=testdata$age,
                     n=nrow(testdata),
                     bmu=bmu0, 
                     bsig = bsig0,
                     lmumean=lmumean0,
                     lmusig=lmusig0)
fit <- stan(file = 'UnmeasuredSelectionBias_Hierarchical.stan', data = stan_dat,
                iter = 2000, thin =5, chains = 2)
samps <- extract(fit)
    
# predict voltage distribution after addressing selection bias ----
dist1 <- gen_dist(samps, simdat)
dist1_summary <- make_distqs(t(dist1))

# estimate reliability ----
req <- 26.8
rel0 <- calc_rel(dist0_summary, req=req)
rel1 <- calc_rel(dist1_summary, req=req)
relt <- 1-length(which(dist_true < 26.8))/length(dist_true)

save(rel0, rel1, relt, file= "rel_est_selection.Rda")


#plot results-------
filename <- "Figure_Battery_SelectionBias_Hierarchical.png"
png(filename, width=1.3*480, height=480)
par(mar=c(5, 5, 1, 1), lwd=3, cex.axis=2, cex.lab=2)
plot(ecdf(sample(dist_true, 700, replace=F)), main="", xlab="Voltage", ylab="Empirical CDF", lwd=7)
lines(ecdf(dist0_summary[2,]), col=rgb(.3, .6, 0, 1))
lines(ecdf(dist0_summary[1,]), col=rgb(.3, .6, 0, .1))
lines(ecdf(dist0_summary[3,]), col=rgb(.3, .6, 0, .1))

lines(ecdf(sample(dist1_summary[2,], 700, replace=F)), col=rgb(0,.3, .6, 1))
 lines(ecdf(sample(dist1_summary[1,], 700, replace=F)), col=rgb(0,.3, .6, .05), lty=2)
 lines(ecdf(sample(dist1_summary[3,], 700, replace=F)), col=rgb(0,.3, .6, .05), lty=2)
 
 abline(v=req, lty=2, lwd=3)
 
legend("bottomright", col=c(1, rgb(.3, .6, 0, 1),rgb(0,.3, .6, 1)), lwd=4, c("Truth", "Naive estimate", "Sens. study for selection bias"), cex=1.5)
dev.off()
