
#############################################
### SENSITIVITY STUDY WITH NO UNCERTAINTY ###
### IN SENSITIVITY PARAMETERS             ###
### SELECTION BIAS EXAMPLE                ###
#############################################

# Load libraries and functions ----
library(rstan)
library(msm)
source("DGM.R")
source("simulator_functions.R")

# Parameters for sensitivity study -----
agepred <- 50 # age to predict to, in units of half-years
ntestdata <- 200 # sample size

# simulated data - correct voltage distribution -----
simdat <- gen_simdat(agepred)
dist_true <- simdat$volt

# generate surveillance data with selection bias ----
testdata <- gen_testdata()

# fit model without addressing selection bias ----
stan_dat <- list(y=testdata$volt,
                 age=testdata$age,
                 n=nrow(testdata))
fit0 <- stan(file = 'Linear_Model.stan', data = stan_dat,
             iter = 2000, thin =5, chains = 2)
samps0 <- extract(fit0)

# predict voltage distribution without addressing selection bias ----
testdata2 <- testdata
testdata2$age <- agepred
dist0 <- t(sapply(1:length(samps0$beta0), function(jj)
  sort(rnorm(nrow(testdata2), samps0$beta0[jj] +samps0$beta1[jj]*testdata2$age, samps0$sigma[jj]))))
dist0_summary <- make_distqs(dist0)


#sensitivity study model addressing selection bias-----
stan_dat <- list(y=testdata$volt,
                   age=testdata$age,
                   n=nrow(testdata),
                   beta2=-5)
fit <- stan(file = 'UnmeasuredSelectionBias.stan', data = stan_dat,
              iter = 2000, thin =5, chains = 2)
samps <- extract(fit)
  
# predict voltage distribution after addressing selection bias ----
dist1 <- t(sapply(1:length(samps$beta0), function(ii)
    rnorm(nrow(simdat), samps$beta0[ii] +samps$beta1[ii]*simdat$age + stan_dat$beta2*simdat$load, samps$sigma[ii])))
dist1_summary <- make_distqs(dist1)


#plot results -----
filename <- "Figure_Battery_SelectionBias_Corrected.png"
png(filename, width=1.3*480, height=480)
par(mar=c(5, 5, 1, 1), lwd=3, cex.axis=2, cex.lab=2)
plot(ecdf(sample(dist_true, 700, replace=F)), main="", xlab="Voltage", ylab="Empirical CDF", 
     lwd=7, col=rgb(.1,.1,.1, .2),col.01line=1)
lines(ecdf(dist0_summary[2,]), col=rgb(.3, .6, 0, 1))

lines(ecdf(sample(dist1_summary[2,], 700, replace=F)), col=rgb(0,.3, .6, .3))

legend("bottomright", col=c(rgb(.1,.1,.1, .8), rgb(.3, .6, 0, 1),rgb(0,.3, .6, 1)), lwd=4, c("Truth", "Naive estimate", "Corrected for selection bias"), cex=1.5)
dev.off()
