
#############################################
### SENSITIVITY STUDY WITH UNCERTAINTY    ###
### IN SENSITIVITY PARAMETERS             ###
### CONFOUNDING BIAS EXAMPLE              ###
#############################################

# Load libraries and functions ----
library(rstan)
library(msm)
source("DGM.R")
source("simulator_functions.R")

# Parameters for sensitivity study -----
agepred <- 50  # age to predict to, in units of half-years
ntestdata <- 200 # sample size

bmu <- -4 # prior mean of beta2
bsig <- 2 # prior standard deviation of beta2
gamma0mu <- .01 # prior mean on age-load trend 
gamma0sig <- .02 # prior standard deviation on age-load trend 

# generate surveillance data with confounding bias ----
simdat <- gen_simdat(agepred)
dist_true <- simdat$volt

# generate testdata 
testdata <- gen_testdata(type="confound")

# fit model without addressing confounding bias ----
stan_dat <- list(y=testdata$volt,
                 age=testdata$age,
                 n=nrow(testdata))
fit0 <- stan(file = 'Linear_Model.stan', data = stan_dat,
             iter = 2000, thin =5, chains = 2)
samps0 <- extract(fit0)

# predict voltage distribution without addressing confounding bias ----
testdata2 <- testdata
testdata2$age <- agepred
dist0 <- t(sapply(1:length(samps0$beta0), function(jj)
  sort(rnorm(nrow(testdata2), samps0$beta0[jj] +samps0$beta1[jj]*testdata2$age, samps0$sigma[jj]))))
dist0_summary <- make_distqs(dist0)

#sensitivity study model addressing confounding bias-----
stan_dat <- list(y=testdata$volt,
                   age=testdata$age,
                   n=nrow(testdata),
                   bmu=bmu,
                   bsig=bsig,
                 gamma0mu=gamma0mu,
                 gamma0sig=gamma0sig)
fit <- stan(file = 'UnmeasuredConfoundingHierarchical.stan', data = stan_dat,
              iter = 2000, thin =5, chains = 2)
samps <- rstan::extract(fit)
  
# predict voltage distribution after addressing confounding bias ----
dist1 <- sapply(1:length(samps$beta0), function(ii)
    rnorm(nrow(simdat), samps$beta0[ii] +samps$beta1[ii]*simdat$age + samps$beta2[ii]*simdat$load, samps$sigma[ii]))
dist1_summary <- make_distqs(t(dist1))

 
# estimate reliability ----
req <- 26.8
rel0 <- calc_rel(dist0_summary, req=req)
rel1 <- calc_rel(dist1_summary, req=req)
relt <- 1-length(which(dist_true < 26.8))/length(dist_true)

save(rel0, rel1, relt, file= "rel_est_confounding.Rda")


#plot results -----
filename <- "Figure_Battery_Confounding_Hierarchical.png"
png(filename, width=1.3*480, height=480)
par(mar=c(5, 5, 1, 1), lwd=3, cex.axis=2, cex.lab=2)
plot(ecdf(sample(dist_true, 700, replace=F)), main="", xlab="Voltage", ylab="Empirical CDF", lwd=7, xlim=quantile(c(dist1, dist0), c(.005, .995)))
lines(ecdf(dist0_summary[2,]), col=rgb(.3, .6, 0, 1))
lines(ecdf(dist0_summary[1,]), col=rgb(.3, .6, 0, .1))
lines(ecdf(dist0_summary[3,]), col=rgb(.3, .6, 0, .1))

lines(ecdf(sample(dist1_summary[2,], 700, replace=F)), col=rgb(0,.3, .6, 1))
lines(ecdf(sample(dist1_summary[1,], 700, replace=F)), col=rgb(0,.3, .6, .05), lty=2)
lines(ecdf(sample(dist1_summary[3,], 700, replace=F)), col=rgb(0,.3, .6, .05), lty=2)

abline(v=req, lty=2, lwd=3)

legend("bottomright", col=c(1, rgb(.3, .6, 0, 1),rgb(0,.3, .6, 1)), lwd=4, c("Truth", "Naive estimate", "Confounding adjustment"), cex=1.5)
dev.off()



